package fr.bhprod.sga.entity;

import java.io.File;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.jme.bounding.BoundingBox;
import com.jme.image.Texture;
import com.jme.math.FastMath;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.Spatial;
import com.jme.scene.TriMesh;
import com.jme.scene.state.TextureState;
import com.jme.util.TextureManager;
import com.jmex.model.animation.KeyframeController;
import com.jmex.model.util.ModelLoader;

import fr.bhprod.sga.terrain.Terrain;
import fr.bhprod.sga.terrain.TerrainPosition;
import fr.bhprod.sga.terrain.TerrainTools;

/**
 * 
 * @author jbeaufils
 * @version 1.0
 */

public class DummyObject implements IObject {
	protected Vector3f position = null;
	protected float angle = 0;
	protected float scale = 1;
	protected Node node = null;
	protected float speed = 1;
	//private ParticleMesh[] particules = null;
	protected boolean isAnimated = false;
	protected HashMap<String,Animation> animations = null;
	public static final Logger LOGGER = Logger.getLogger(DummyObject.class);
	
	protected DummyObject() {
	}

	public DummyObject(String nodeName, Vector3f pos, File xmlModelFile, Renderer renderer) {
		node = new Node(nodeName);

		XmlModel model = XmlModelHandler.load(xmlModelFile);
		
		node = ModelLoader.loadModel(new File(model.getFile()));

		node.setName(nodeName);
		for (Iterator<Spatial> it = node.getChildren().iterator(); it.hasNext();) {
			it.next().setName(nodeName);
		}
		
		TextureState ts = renderer.createTextureState();
		ts.setEnabled(true);
		Texture texture = TextureManager.loadTexture(model.getTextureFile(), Texture.MM_LINEAR_LINEAR, Texture.FM_LINEAR );
		texture.setWrap( Texture.WM_WRAP_S_WRAP_T );
		ts.setTexture(texture);
		node.setRenderState(ts);

		setScale(model.getScale());
		
		node.setModelBound(new BoundingBox());
		node.updateModelBound();

		setAnimated(model.isAnimated());
		
		if (model.isAnimated()) {
			animations = new HashMap<String,Animation>(model.getAnimations());
		}

		setSpeed(model.getSpeed());

		setPosition(pos);
		
		// TODO : A charger depuis le fichier
		//node.setLocalRotation(new Quaternion().fromAngleAxis(FastMath.PI/2, new Vector3f(0,1,0)));
		//setAngle(0);
		angle = 90 * FastMath.DEG_TO_RAD;
	}
	
	public void setScale(float scale) {
		this.scale = scale;
		node.setLocalScale(scale);
	}

	public void setPosition(Vector3f position) {
		this.position = position;
		node.setLocalTranslation(position);
	}
	
	public void putOnTerrain(Vector3f pPosition, Terrain terrain, Renderer renderer) {
		LOGGER.debug("putOnTerrain - Entry");
		TerrainPosition pos = TerrainTools.getTerrainPosition(terrain, pPosition);
		setPosition(new Vector3f(pos.getPosition().x, pos.getPosition().y + getBoundingBox().yExtent, pos.getPosition().z));
		LOGGER.debug("putOnTerrain - Exit");
	}

	public BoundingBox getBoundingBox() {
		return (BoundingBox)((TriMesh)(node.getChild(0))).getWorldBound();
	}

	public void setSpeed(float speed) {
		this.speed = speed;
		if (isAnimated) {
			getKeyframeController().setSpeed(speed);
		}
	}

	public KeyframeController getKeyframeController() {
		if (isAnimated) {
			return (KeyframeController) getNode().getChild(0).getController(0);
		} else {
			return null;
		}
	}
	
	public void setAnimation(String animName) throws IllegalArgumentException, IllegalStateException {
		if (!isAnimated) {
			throw new IllegalStateException("This model is not animated");
		}

		Animation anim = animations.get(animName);
		if (anim == null) {
			throw new IllegalArgumentException("The animation ("+animName+") doesn't exist for this model");
		}
		getKeyframeController().setNewAnimationTimes(anim.getStartFrame(), anim.getEndFrame());
	}
	
	public void setAnimationCycle(boolean cycle) throws IllegalStateException {
		if (!isAnimated) {
			throw new IllegalStateException("This model is not animated");
		}

		if (cycle == true) {
			getKeyframeController().setRepeatType(KeyframeController.RT_CYCLE);
		} else {
			getKeyframeController().setRepeatType(KeyframeController.RT_WRAP);
		}
	}


	
	
	public Vector3f getPosition() {
		return position;
	}
	
	public Node getNode() {
		return node;
	}
	
	public HashMap<String, Animation> getAnimations() {
		return animations;
	}
	
	public float getSpeed() {
		return speed;
	}
	
	public float getScale() {
		return scale;
	}

	public boolean isAnimated() {
		return isAnimated;
	}

	public void setAnimated(boolean isAnimated) {
		this.isAnimated = isAnimated;
	}

	public float getAngle() {
		return angle;
	}

	public void setAngle(float angle) {
		this.angle = angle;
		node.setLocalRotation(new Quaternion().fromAngleAxis(angle, new Vector3f(0,1,0)));
	}
}

