package fr.bhprod.sga.entity;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.jme.math.FastMath;
import com.jme.math.Matrix4f;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.shape.Box;
import com.jme.scene.shape.Sphere;
import com.jme.scene.state.LightState;

import fr.bhprod.sga.path.AStar;
import fr.bhprod.sga.terrain.Terrain;

/**
 * 
 * @author jbeaufils
 * @version 1.0
 */

public class UnitObject extends PlayableObject implements IMovable, IUnit {
	private float destAngle = 0;
	private Vector3f destination = null;
	private boolean isRotating = false;
	private boolean isMoving = false;
	private Terrain terrain = null;
	private Renderer renderer = null;
	private Vector<int[]> waypoints = null;
	private long oldTime = 0;
	public static final Logger LOGGER = Logger.getLogger(UnitObject.class);

	public UnitObject(String nodeName, Vector3f pos, File xmlModelFile, Terrain terrain, Renderer renderer) {
		super(nodeName, pos, xmlModelFile, renderer);
		this.terrain = terrain;
		this.renderer = renderer;
	}
	
	// TODO : Dplacement avec animation (cf. IMovable)
	// TODO : Caractristiques : points de vie, ...
	// TODO : Actions possibles
	// TODO : Peut appartenir  une quipe

	
	public void setDestination(Vector3f dest, long time, Node rootNode) {
		destination = new Vector3f(dest);
LOGGER.debug("DESTINATION :"+destination);
LOGGER.debug("ANGLE AVANT :"+(angle*FastMath.RAD_TO_DEG));
		Vector3f posToDest = new Vector3f(dest.x - getPosition().x, 0, dest.z - getPosition().z).normalize();
		
Sphere sphere1 = new Sphere("sphere1",new Vector3f(getPosition().x, 20, getPosition().z-2), 10, 10, 10);
sphere1.setSolidColor(ColorRGBA.blue);
sphere1.updateRenderState();
node.attachChild(sphere1);
Box box1 = new Box("box1",new Vector3f(dest.x, 20, dest.z), 0.5f, 0.5f, 0.5f);
box1.setSolidColor(ColorRGBA.red);
box1.updateRenderState();
rootNode.attachChild(box1);
destAngle = posToDest.normalize().angleBetween(Vector3f.UNIT_X);
LOGGER.debug("ANGLE APRES :"+(destAngle*FastMath.RAD_TO_DEG));
		isRotating = true;
		isMoving = true;
		setAnimation("run");
		oldTime = time;
	}
	
	public void move(long time, Node rootNode) {
		// If an update is needed

		/*
		 * 1) Stand
		 * 2) Clic sur le terrain -> setDestination
		 * 3) Dmarrage de la rotation + run -> Tant que la direction du modle est diffrente de vers la destination
		 * 4) Dmarrage du dplacement + run ... -> Tant que la distance entre le modle et la destination est > D
		 * 5) Stand
		 */
		
		if (isRotating) {
			// First, test if the model must turn toward the destination
			//LOGGER.debug("model.update : angle = "+(angle*FastMath.RAD_TO_DEG)+", destAngle = "+(destAngle*FastMath.RAD_TO_DEG));
			float diff = destAngle - angle;
			if (FastMath.abs(diff) <= 0.05) {
				angle = destAngle;
        		destAngle = 0;
        		isRotating = false;
    			LOGGER.debug("model.update : final angle = "+(angle*FastMath.RAD_TO_DEG));
			} else {
// TODO : Grer une vitesse de rotation externalise (<> de 0.01) : voir vitesse de l'objet
				if (diff < 0) {
					angle -= 0.05f;
				} else {
					angle += 0.05f;
				}
				getNode().setLocalRotation(new Quaternion().fromAngleAxis(angle, new Vector3f(0,1,0)));
        	}
		} else if (isMoving) {
			// Then, test if the model must move toward the destination
			// FIXME : a finir
			LOGGER.debug("\n\nPOSITION : "+position);
			position.distance(destination);
			
	        float dx = position.x - destination.x;
	        float dz = position.z - destination.z;
	        float currentSquaredDistance = dx * dx + dz * dz;
			LOGGER.debug("CURRENT DISTANCE : "+currentSquaredDistance);
			
			LOGGER.debug("OLD TIME : "+oldTime);
			LOGGER.debug("NEW TIME : "+time);
			LOGGER.debug("DIRECTION : "+getDirection());

			Vector3f newPosition = new Vector3f(position);
			newPosition = newPosition.add(getDirection().mult((speed/1000) * (time - oldTime)));
			LOGGER.debug("DESTINATION : "+destination);
			LOGGER.debug("OLD POSITION : "+position);
			LOGGER.debug("NEW POSITION : "+newPosition);
			
	        dx = position.x - newPosition.x;
	        dz = position.z - newPosition.z;
	        float futureSquaredDistance = dx * dx + dz * dz;
			LOGGER.debug("FUTURE DISTANCE : "+futureSquaredDistance);
			
			// If we have reach the destination
			if (futureSquaredDistance >= currentSquaredDistance) {
				LOGGER.debug("AT DESTINATION !");
				newPosition.set(destination);
				waypoints.removeElementAt(0);
				if (waypoints.isEmpty()) {
					destination = null;
					setAnimation("stand");
					isMoving = false;
					waypoints = null;
				} else {
					Vector3f destination = new Vector3f(waypoints.get(0)[0] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2), 20, -(waypoints.get(0)[1] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2)));
					setDestination(destination, time, rootNode);
				}
			}
			
			putOnTerrain(newPosition, terrain, renderer);

			if (selectionShape != null) {
				destroySelectionShape();
				createSelectionShape(terrain, rootNode);
			}
		}

		oldTime = time;
	}
	
	public boolean isMoving() {
		return isMoving;
	}
	
	public Vector3f getDirection() {
		Matrix4f matRot = new Matrix4f();
		matRot.fromAngleAxis(angle, new Vector3f(0,1,0));
		Vector3f vec = new Vector3f(Vector3f.UNIT_X);
		matRot.rotateVect(vec);
		return vec;
	}


	
	public void goToDest(Vector3f dest, long time, Terrain terrain, Node rootNode) {
		int startTilex = (int)(position.x / terrain.getTileRealSize());
		int startTilez = (int)(-position.z / terrain.getTileRealSize());
		
		int endTilex = (int)(dest.x / terrain.getTileRealSize());
		int endTilez = (int)(-dest.z / terrain.getTileRealSize());
		
		waypoints = AStar.calculateBasicAStar(startTilex, startTilez, endTilex, endTilez, terrain.getTilesX(), terrain.getTilesZ(), terrain.getInpassMap(), rootNode);
		
		Sphere sphere = new Sphere("jhbjh", new Vector3f(endTilex * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2), 10, -(endTilez * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2))), 20, 20, 0.5f);
		sphere.setLightCombineMode(LightState.OFF);
		sphere.setSolidColor(ColorRGBA.green);
		rootNode.attachChild(sphere);
		rootNode.updateRenderState();
		
		if (waypoints != null && !waypoints.isEmpty()) {
			for (Iterator<int[]> it = waypoints.iterator(); it.hasNext();) {
				int[] values = it.next();
				sphere = new Sphere("jhbjh"+values[0]+values[1], new Vector3f(values[0] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2), 20, -(values[1] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2))), 20, 20, 0.5f);
				rootNode.attachChild(sphere);
			}
		}

		Vector3f destination = new Vector3f(waypoints.get(0)[0] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2), 20, -(waypoints.get(0)[1] * terrain.getTileRealSize() + (terrain.getTileRealSize() / 2)));
		setDestination(destination, time, rootNode);
	}
}

