package fr.bhprod.sga.tools;

import com.jme.math.Matrix4f;
import com.jme.math.Vector3f;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.TriMesh;
import com.jme.scene.state.WireframeState;
import com.jme.util.geom.BufferUtils;
import com.jmex.effects.ProjectedTextureUtil;

/**
 * Create a mesh for a projector frustum.
 * 
 * @author jbeaufils
 * @version 1.0
 */
public class DisplayDecalProjectorNode extends Node {
	/**
	 * serializable id.
	 */
	private static final long serialVersionUID = 974695001050041672L;

	/**
	 * Create the mesh.
	 * 
	 * @param renderer
	 *            The current renderer
	 * @param name
	 *            The name of the node
	 * @param position
	 *            The position of the projector
	 * @param aim
	 *            The position the projector look at
	 * @param up
	 *            The up vector
	 * @param left
	 *            Left of the frustum
	 * @param right
	 *            Right of the frustum
	 * @param top
	 *            Top of the frustum
	 * @param bottom
	 *            Bottom of the frustum
	 * @param distance
	 *            Distance of the frustum projection
	 */
	public DisplayDecalProjectorNode(final Renderer renderer, final String name,
			final Vector3f position, final Vector3f aim, final Vector3f up,
			final float left, final float right, final float top,
			final float bottom, final float distance) {
		super(name);

		Matrix4f matRot = new Matrix4f();
		ProjectedTextureUtil.matrixLookAt(position, aim, up, matRot);

		Vector3f topLeft = new Vector3f(left, top, -distance);
		topLeft = matRot.mult(topLeft).add(position);
		Vector3f topRight = new Vector3f(right, top, -distance);
		topRight = matRot.mult(topRight).add(position);
		Vector3f bottomLeft = new Vector3f(left, bottom, -distance);
		bottomLeft = matRot.mult(bottomLeft).add(position);
		Vector3f bottomRight = new Vector3f(right, bottom, -distance);
		bottomRight = matRot.mult(bottomRight).add(position);

		WireframeState ws = renderer.createWireframeState();
		ws.setFace(WireframeState.WS_FRONT_AND_BACK);

		TriMesh cam = new TriMesh("cam", BufferUtils
				.createFloatBuffer(new Vector3f[] { position, topLeft,
						topRight, bottomLeft, bottomRight }), null, null, null,
				BufferUtils.createIntBuffer(new int[] { 0, 1, 2, 0, 2, 4, 0, 4,
						3, 0, 3, 1 }));
		cam.setRenderState(ws);
		attachChild(cam);
	}
}
